import { makeStyles } from "@material-ui/core/styles";
import { useState, useEffect, useRef, useCallback } from "react";
import { useSelector,useDispatch } from "react-redux";
import shortid from 'shortid';

import SendMessage from "./SendMessage";
import MessageLeft from './Messages/MessageLeft'
import MessageRight from './Messages/MessagesRight'
import AlertInfo from "../../../reusableComponents/AlertInfo";
import { getMessages } from '../../../../redux/messages/selector'
import { getNumber } from '../../../../redux/authorization/selector'
import { getChat } from '../../../../redux/chat/selector'
import { getScroll } from '../../../../redux/control/selector'
import { actionScroll } from '../../../../redux/control/action'
import { asyncGetMessagesById } from '../../../../redux/messages/operations'
import { seenChat } from "../../../../api-data";
const debounce = require('lodash.debounce');

const useStyles = makeStyles({   
    container: {
      width: "100%",
      height: "100%",
      maxWidth: "100%",
      maxHeight: "100%",
      display: "flex",
      alignItems: "center",
      alignContent:"center",
      flexDirection: "column",
      position: "relative",
      overflowY: "scroll",
      paddingTop: 60,
    },
    messagesBody: {
      width: "40%",
      height: "80%",
    },   
});

const ChatBar = () => {
  const classes = useStyles();
  const dispatch = useDispatch()
  const messages = useSelector(getMessages)
  const userNumber = useSelector(getNumber)
  const { companionId } = useSelector(getChat)
  const scroll = useSelector(getScroll)
  const [isArrow, setIsArrow] = useState<boolean>(false)
  const divRef = useRef<any | null>(null)
  const handleScrollTo = () => {
     divRef.current&&divRef.current.scrollTo({
     top: divRef.current.scrollHeight,
     behavior: 'smooth'
     })
   }
  const handleScroll = ({ target }: any) => {
    const different = target.scrollHeight - target.scrollTop
    if (different < 900) seenChat(companionId)
    setIsArrow(different < 1500 ? false : true)
  }
  const debouncedHandleScroll = useCallback(debounce(handleScroll, 300), []);

  useEffect(() => {
    if (scroll) {
      dispatch(asyncGetMessagesById(companionId, handleScrollTo))
      dispatch(actionScroll(false))
    }
  }, [dispatch,scroll, companionId])  
  
  useEffect(() => {
    dispatch(asyncGetMessagesById(companionId, handleScrollTo))
    const handleReset = () => dispatch(asyncGetMessagesById(companionId,null))
    const idInterval = setInterval(handleReset, 3000);
    return () => clearInterval(idInterval);
  }, [dispatch, companionId]);


  return (
      <div ref={divRef} className={classes.container} onScroll={debouncedHandleScroll}>
        <div  className={classes.messagesBody}>
        {messages.length > 0 ? messages.map(({message,avatarUrl,name,lastName,color,updatedAt,number}:any,i:number) => {
          if (number !== userNumber) {
            return (
            <MessageLeft
            key={shortid.generate()}
            message={message}
            updatedAt={updatedAt}
            avatarUrl={avatarUrl}
            name={name}
            lastName={lastName}
            color={color}
          />)
          } else {
            return (
            <MessageRight
            key={shortid.generate()}    
            message={message}
            updatedAt={updatedAt}
            avatarUrl={avatarUrl}
            name={name}
            lastName={lastName}
            color={color}    
          />)}}):<AlertInfo name='You do not have messages yet!' />}
      </div>         
      <SendMessage isArrow={isArrow} handleScrollTo={handleScrollTo}/>   
      </div>
  );
}

export default ChatBar